# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2011 Justin Santa Barbara
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""
Tests For Topology
"""

from nova import test
from nova.scheduler import topology


class NamedTopologyTestCase(test.TestCase):
    """Test case for named topology"""
    def _build_simple_topology(self):
        t = topology.NamedTopology()
        t.add(topology.TopologyNode('room1.rack1.machine1'))
        t.add(topology.TopologyNode('room1.rack1.machine2'))
        t.add(topology.TopologyNode('room1.rack2.machine1'))
        t.add(topology.TopologyNode('room1.rack2.machine2'))
        t.add(topology.TopologyNode('room2.rack1.machine1'))
        t.add(topology.TopologyNode('room2.rack1.machine2'))
        t.add(topology.TopologyNode('room2.rack2.machine1'))
        t.add(topology.TopologyNode('room2.rack2.machine2'))
        self.topology = t

    def _distance(self, key1, key2):
        node1 = self.topology.get_node_by_key(key1)
        node2 = self.topology.get_node_by_key(key2)
        return self.topology.distance(node1, node2)

    def _keys_by_distance_from(self, key1):
        node1 = self.topology.get_node_by_key(key1)
        nodes = self.topology.find_by_distance_from(node1)
        return [node.key for node in nodes]

    def test_distances(self):
        self._build_simple_topology()

        self.assertEquals(0, self._distance('room1.rack1.machine1',
                                            'room1.rack1.machine1'))

        self.assertEquals(1, self._distance('room1.rack1.machine1',
                                            'room1.rack1.machine2'))

        self.assertEquals(2, self._distance('room1.rack1.machine1',
                                            'room1.rack2.machine1'))

        self.assertEquals(3, self._distance('room1.rack1.machine1',
                                            'room2.rack1.machine1'))

    def test_order_by_distance(self):
        self._build_simple_topology()

        keys = self._keys_by_distance_from('room1.rack1.machine1')
        # We assume a stable sort...
        self.assertEquals(keys, ['room1.rack1.machine1',
                                 'room1.rack1.machine2',
                                 'room1.rack2.machine1',
                                 'room1.rack2.machine2',
                                 'room2.rack1.machine1',
                                 'room2.rack1.machine2',
                                 'room2.rack2.machine1',
                                 'room2.rack2.machine2',
                                 ])

        keys = self._keys_by_distance_from('room2.rack2.machine2')
        # We assume a stable sort...
        self.assertEquals(keys, ['room2.rack2.machine2',
                                 'room2.rack2.machine1',
                                 'room2.rack1.machine1',
                                 'room2.rack1.machine2',
                                 'room1.rack1.machine1',
                                 'room1.rack1.machine2',
                                 'room1.rack2.machine1',
                                 'room1.rack2.machine2',
                                 ])
